-- < minnie.e > - mouse routines (mini version of Pete Eberlein's mighty.e)
-- minnie does not redefine pixel() and get_pixel()
-- edited by Colin Taylor - cetaylor@compuserve.com
-- version 1.0 - 7/31/99

-- minnie's commands:
--   mouse_pointer() - similar to Euphoria's mouse_pointer() command
--   poll_mouse() - gets the mouse location and button status
--   set_mouse_cursor() - changes the mouse pointer image

-- minnie assumes that:
--   - the computer is in a valid graphics mode
--   - a mouse is installed

include machine.e
include graphics.e

sequence m_back, m_pos, m_map, m_mask, m_hotspot, m_bord, m_lastpos
integer m_visible, m_buts, m_moved, mWide, mTall, extended_mouse

constant DEFAULT = {{  -- standard arrow cursor
    "           ",
    " /         ",
    " //        ",
    " ///       ",
    " ////      ",
    " /////     ",
    " //////    ",
    " ///////   ",
    " ////////  ",
    " /////     ",
    " // //     ",
    " /   //    ",
    "     //    ",
    "      //   ",
    "      //   ",
    "           "}-32,
        #FF*(' '={
    "xx         ",
    "x/x        ",
    "x//x       ",
    "x///x      ",
    "x////x     ",
    "x/////x    ",
    "x//////x   ",
    "x///////x  ",
    "x////////x ",
    "x/////xxxxx",
    "x//x//x    ",
    "x/x x//x   ",
    "xx  x//x   ",
    "x    x//x  ",
    "     x//x  ",
    "      xxx  "}),
    {0,0}}

extended_mouse = 0

global procedure mouse_pointer(integer show_it)
-- shows (1) or hides (0) the mouse pointer
    sequence vparams, regs, pos
    if not extended_mouse then
        -- initialize minnie mouse
        regs = repeat(0, 10)
        regs = dos_interrupt(#33, regs) -- reset mouse
        extended_mouse = 1
        vparams = video_config()
        m_bord = {vparams[VC_XPIXELS], vparams[VC_YPIXELS]} - 1
        m_pos = floor((m_bord+1)/2)
        m_back = {{}}
        m_visible = 1  -- not yet visible
        m_buts = 0     -- no buttons pressed
        m_map = DEFAULT[1]
        m_mask = DEFAULT[2]
        m_hotspot = DEFAULT[3]
        mWide = length(m_map[1])
        mTall = length(m_map)
    end if
    if show_it then
        if m_visible > 0 then
            m_visible = m_visible - 1
        end if
        if m_visible = 0 then
            m_back = repeat(0, mTall)
            pos = (m_pos - m_hotspot) & mWide
            for y = 1 to mTall do
                m_back[y] = get_pixel(pos)
                pos[2] = pos[2] + 1
            end for
            pos = m_pos - m_hotspot
            for y = 1 to mTall do
                pixel(xor_bits(and_bits(m_back[y], m_mask[y]), m_map[y]), pos)
                pos[2] = pos[2] + 1
            end for
        end if
    else
        if m_visible = 0 then
            pos = m_pos - m_hotspot
            for y = 1 to length(m_back) do
                pixel(m_back[y], pos)
                pos[2] = pos[2] + 1
            end for
        end if
        m_visible = m_visible + 1
    end if
end procedure  -- mouse_pointer

global function poll_mouse()
-- returns {buttons, x_pos, y_pos} of the mouse
-- bit masks for buttons: 1-left 2-right 4-middle
    sequence regs, temp, pos
    integer dy, dx, bx1, bx2, bx3, bx4, tx1, tx2
    regs = repeat(0, 10)
    regs[REG_AX] = #B              -- get relative motion counters
    regs = dos_interrupt(#33, regs)
    temp = regs[REG_CX] & regs[REG_DX]
    m_lastpos = m_pos                
    m_pos = m_pos + temp - 65536 * (temp > 32767)
    m_pos = m_pos * (m_pos >= 0) * (m_pos <= m_bord)
             + m_bord * (m_pos > m_bord)
    regs[REG_AX] = 3               -- get mouse buttons in BX
    regs = dos_interrupt(#33, regs)
    m_moved = (m_lastpos[1] != m_pos[1]) or (m_lastpos[2] != m_pos[2])
    if m_moved then
        if m_visible = 0 then
            temp = repeat(0, mTall)
            pos = (m_pos - m_hotspot) & mWide
            for y = 1 to mTall do
                temp[y] = get_pixel(pos)
                pos[2] = pos[2] + 1
            end for
            dx = m_pos[1] - m_lastpos[1]
            dy = m_pos[2] - m_lastpos[2]
            if (dy > -mTall and dy < mTall) 
            and (dx > -mWide and dx < mWide) then
                if dx >= 0 and dx < mWide then   -- mouse moved to the right
                    tx1 = 1           tx2 = mWide - dx
                    bx1 = 1 + dx      bx2 = mWide
                    bx3 = 1           bx4 = dx
                    dx = 0
                elsif dx > -mWide and dx < 0 then -- mouse moved to the left
                    tx1 = 1 - dx      tx2 = mWide
                    bx1 = 1           bx2 = mWide + dx
                    bx3 = mWide+1+dx  bx4 = mWide
                    dx = mWide + dx
                end if
                dy = - dy
                pos = m_lastpos - m_hotspot
                for y = 1 to length(m_back) do
                    dy = dy + 1
                    if dy >= 1 and dy <= mTall then
                        temp[dy][tx1..tx2] = m_back[y][bx1..bx2]
                        pixel(m_back[y][bx3..bx4], pos+{dx,y-1})
                    else
                        pixel(m_back[y], pos+{0,y-1})
                    end if
                end for
            else  -- new mouse image doesn't overlap old background
                pos = m_lastpos-m_hotspot
                for y = 1 to length(m_back) do
                    pixel(m_back[y], pos)
                    pos[2] = pos[2] + 1
                end for
            end if
            pos = m_pos-m_hotspot
            for y = 1 to mTall do
                pixel(xor_bits(and_bits(temp[y], m_mask[y]), m_map[y]), pos)
                pos[2] = pos[2] + 1
            end for
            m_back = temp
        end if
    end if
    return regs[REG_BX] & m_pos
end function  -- poll_mouse
 
global function mouse_inside(integer x1, integer y1, integer x2, integer y2)
    return m_pos[1] >= x1
       and m_pos[2] >= y1
       and m_pos[1] <= x2
       and m_pos[2] <= y2
end function

global function mouse_enter(integer x1, integer y1, integer x2, integer y2)
    return m_pos[1] >= x1
       and m_pos[2] >= y1
       and m_pos[1] <= x2
       and m_pos[2] <= y2
       and (m_lastpos[1] < x1 
       or m_lastpos[2] < y1 
       or m_lastpos[1] > x2 
       or m_lastpos[2] > y2)
end function

global function mouse_leave(integer x1, integer y1, integer x2, integer y2)
    return m_lastpos[1] >= x1
       and m_lastpos[2] >= y1
       and m_lastpos[1] <= x2
       and m_lastpos[2] <= y2
       and (m_pos[1] < x1 
       or m_pos[2] < y1 
       or m_pos[1] > x2 
       or m_pos[2] > y2)
end function

global procedure set_mouse_cursor(sequence cursor)
-- cursor = {map, mask, hotspot}, or {} for default cursor
-- mask is AND'ed to screen, then map is XOR'ed on top
--  (map and mask must be exactly the same size)
-- hotspot is {dx,dy} displacement of mouse relative to image
    if length(cursor) = 0 then
        cursor = DEFAULT
    end if
    mouse_pointer(0)
    m_map = cursor[1]
    m_mask = cursor[2]
    m_hotspot = cursor[3]
    mWide = length(m_map[1])
    mTall = length(m_map)
    mouse_pointer(1)
end procedure  -- set_mouse_cursor

